@@ -202,7 +202,7 @@ public:
 
 private:
 	TokenIt _id; ///< Identifier of the string
-	TokenIt _in_symbol; ///< Token holding "at"
+	TokenIt _in_symbol; ///< Token holding "in"
 	Expression::Ptr _range; ///< Range expression
 };
 
@@ -348,6 +348,7 @@ public:
 		: _expr(std::forward<ExpPtr>(expr))
 	{
 		_id = _tokenStream->emplace_back(TokenType::STRING_LENGTH, std::forward<Str>(id));
+		_tokenStream->moveAppend(_expr->getTokenStream());
 	}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -379,6 +380,12 @@ private:
 	Expression::Ptr _expr; ///< Index expression if any
 };
 
+enum class UnaryOperatorPlacement
+{
+    Left,
+    Right
+};
+
 /**
  * Abstract class representing some unary operation.
  */
@@ -388,9 +395,17 @@ public:
 	virtual std::string getText(const std::string& indent = std::string{}) const override
 	{
 		if (_op->getType() == TokenType::NOT || _op->getType() == TokenType::DEFINED)
+		{
+			assert(_operatorPlacement == UnaryOperatorPlacement::Left);
 			return _op->getString() + " " + _expr->getText(indent);
+		}
 		else
-			return _op->getString() + _expr->getText(indent);
+		{
+			if (_operatorPlacement == UnaryOperatorPlacement::Left)
+				return _op->getString() + _expr->getText(indent);
+			else
+				return _expr->getText(indent) + _op->getString();
+		}
 	}
 
 	const Expression::Ptr& getOperand() const { return _expr; }
@@ -400,23 +415,36 @@ public:
 
 protected:
 	template <typename ExpPtr>
-	UnaryOpExpression(TokenIt op, ExpPtr&& expr)
+	UnaryOpExpression(TokenIt op, ExpPtr&& expr, UnaryOperatorPlacement operatorPlacement)
 		: _op(op)
 		, _expr(std::forward<ExpPtr>(expr))
+		, _operatorPlacement(operatorPlacement)
 	{
 	}
 	template <typename ExpPtr>
-	UnaryOpExpression(const std::string& op, TokenType type, ExpPtr&& expr)
+	UnaryOpExpression(const std::string& op, TokenType type, ExpPtr&& expr, UnaryOperatorPlacement operatorPlacement)
 		: _expr(std::forward<ExpPtr>(expr))
+		, _operatorPlacement(operatorPlacement)
 	{
 		_op = _tokenStream->emplace_back(type, op);
 	}
-	virtual TokenIt getFirstTokenIt() const override { return _op; }
-	virtual TokenIt getLastTokenIt() const override { return _expr->getLastTokenIt(); }
+	virtual TokenIt getFirstTokenIt() const override {
+		if (_operatorPlacement == UnaryOperatorPlacement::Left)
+			return _op;
+		else
+			return _expr->getFirstTokenIt();
+	}
+	virtual TokenIt getLastTokenIt() const override {
+		if (_operatorPlacement == UnaryOperatorPlacement::Left)
+			return _expr->getLastTokenIt();
+		else
+			return _op;
+	}
 
 private:
 	TokenIt _op; ///< Unary operation symbol, std::string
 	Expression::Ptr _expr; ///< Expression to apply operator on
+	UnaryOperatorPlacement _operatorPlacement; ///< Determines if the operator should be printed before the expression
 };
 
 /**
@@ -431,7 +459,27 @@ class NotExpression : public UnaryOpExpression
 {
 public:
 	template <typename ExpPtr>
-	NotExpression(TokenIt op, ExpPtr&& expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr)) {}
+	NotExpression(TokenIt op, ExpPtr&& expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr), UnaryOperatorPlacement::Left) {}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+};
+
+/**
+ * Class representing percentual operation.
+ *
+ * For example:
+ * @code
+ * 25%
+ * @endcode
+ */
+class PercentualExpression : public UnaryOpExpression
+{
+public:
+	template <typename ExpPtr>
+	PercentualExpression(ExpPtr&& expr, TokenIt op) : UnaryOpExpression(op, std::forward<ExpPtr>(expr), UnaryOperatorPlacement::Right) {}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -451,7 +499,7 @@ class DefinedExpression : public UnaryOpExpression
 {
 public:
 	template<typename ExpPtr>
-	DefinedExpression(TokenIt op, ExpPtr &&expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr)) {}
+	DefinedExpression(TokenIt op, ExpPtr &&expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr), UnaryOperatorPlacement::Left) {}
 
 	virtual VisitResult accept(Visitor *v) override
 	{
@@ -472,7 +520,7 @@ class UnaryMinusExpression : public UnaryOpExpression
 {
 public:
 	template <typename ExpPtr>
-	UnaryMinusExpression(TokenIt op, ExpPtr&& expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr)) {}
+	UnaryMinusExpression(TokenIt op, ExpPtr&& expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr), UnaryOperatorPlacement::Left) {}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -493,7 +541,7 @@ class BitwiseNotExpression : public UnaryOpExpression
 {
 public:
 	template <typename ExpPtr>
-	BitwiseNotExpression(TokenIt op, ExpPtr&& expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr)) {}
+	BitwiseNotExpression(TokenIt op, ExpPtr&& expr) : UnaryOpExpression(op, std::forward<ExpPtr>(expr), UnaryOperatorPlacement::Left) {}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -728,6 +776,26 @@ public:
 	}
 };
 
+/**
+ * Class representing icontains case-insensitive operation on two strings.
+ *
+ * For example:
+ * @code
+ * pe.sections[0] icontains "text"
+ * @endcode
+ */
+class IcontainsExpression : public BinaryOpExpression
+{
+public:
+	template <typename ExpPtr1, typename ExpPtr2>
+	IcontainsExpression(ExpPtr1&& left, TokenIt op, ExpPtr2&& right) : BinaryOpExpression(std::forward<ExpPtr1>(left), op, std::forward<ExpPtr2>(right)) {}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+};
+
 /**
  * Class representing contains operation on string and regular expression.
  *
@@ -748,6 +816,86 @@ public:
 	}
 };
 
+/**
+ * Class representing startswith operation for string starting with relation.
+ *
+ * For example:
+ * @code
+ * pe.sections[0].name startswith ".t"
+ * @endcode
+ */
+class StartsWithExpression : public BinaryOpExpression
+{
+public:
+	template <typename ExpPtr1, typename ExpPtr2>
+	StartsWithExpression(ExpPtr1&& left, TokenIt op, ExpPtr2&& right) : BinaryOpExpression(std::forward<ExpPtr1>(left), op, std::forward<ExpPtr2>(right)) {}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+};
+
+/**
+ * Class representing istartswith operation for case-insensitive string starting with relation.
+ *
+ * For example:
+ * @code
+ * pe.sections[0].name istartswith ".t"
+ * @endcode
+ */
+class IstartsWithExpression : public BinaryOpExpression
+{
+public:
+	template <typename ExpPtr1, typename ExpPtr2>
+	IstartsWithExpression(ExpPtr1&& left, TokenIt op, ExpPtr2&& right) : BinaryOpExpression(std::forward<ExpPtr1>(left), op, std::forward<ExpPtr2>(right)) {}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+};
+
+/**
+ * Class representing endswith operation for string ending with relation.
+ *
+ * For example:
+ * @code
+ * pe.sections[0].name endswith "xt"
+ * @endcode
+ */
+class EndsWithExpression : public BinaryOpExpression
+{
+public:
+	template <typename ExpPtr1, typename ExpPtr2>
+	EndsWithExpression(ExpPtr1&& left, TokenIt op, ExpPtr2&& right) : BinaryOpExpression(std::forward<ExpPtr1>(left), op, std::forward<ExpPtr2>(right)) {}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+};
+
+/**
+ * Class representing iendswith operation for case-insensitive string ending with relation.
+ *
+ * For example:
+ * @code
+ * pe.sections[0].name iendswith ".t"
+ * @endcode
+ */
+class IendsWithExpression : public BinaryOpExpression
+{
+public:
+	template <typename ExpPtr1, typename ExpPtr2>
+	IendsWithExpression(ExpPtr1&& left, TokenIt op, ExpPtr2&& right) : BinaryOpExpression(std::forward<ExpPtr1>(left), op, std::forward<ExpPtr2>(right)) {}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+};
+
 /**
  * Class representing iequals operation for case-insensitive string compare.
  *
@@ -1182,6 +1330,16 @@ private:
  * @code
  * all of ($str1, $str2)
  * @endcode
+ * 
+ * There can be also "in" with range:
+ * @code
+ * any of ($str1, $str2) in (0..10)
+ * @endcode
+ * 
+ * Or "at" with offset (new in YARA 4.3):
+ * @code
+ * any of ($str1, $str2) at 100
+ * @endcode
  */
 class OfExpression : public ForExpression
 {
@@ -1192,6 +1350,18 @@ public:
 	template <typename ExpPtr1, typename ExpPtr2>
 	OfExpression(ExpPtr1&& forExpr, TokenIt of, ExpPtr2&& set)
 		: ForExpression(std::forward<ExpPtr1>(forExpr), of, std::forward<ExpPtr2>(set))
+		, _location_symbol(std::nullopt)
+		, _location(nullptr)
+	{
+	}
+	/**
+	 * Constructor
+	 */
+	template <typename ExpPtr1, typename ExpPtr2, typename ExpPtr3>
+	OfExpression(ExpPtr1&& forExpr, TokenIt of, ExpPtr2&& set, TokenIt location_symbol, ExpPtr3&& location)
+		: ForExpression(std::forward<ExpPtr1>(forExpr), of, std::forward<ExpPtr2>(set))
+		, _location_symbol(location_symbol)
+		, _location(std::forward<ExpPtr3>(location))
 	{
 	}
 
@@ -1202,11 +1372,35 @@ public:
 
 	virtual std::string getText(const std::string& indent = std::string{}) const override
 	{
-		return _forExpr->getText(indent) + " " + _of_in->getString() + " " + _iterable->getText(indent);
+		std::string output = _forExpr->getText(indent) + " " + _of_in->getString() + " " + _iterable->getText(indent);
+		if (_location && _location_symbol.has_value())
+			output +=  " " + _location_symbol.value()->getString() + " " + _location->getText(indent);
+		return output;
 	}
 
+	/**
+	 * Getter for location expression
+	 * @return Return location expression
+	 */
+	const Expression::Ptr& getLocationExpression() const { return _location; }
+	void setLocationExpression(const Expression::Ptr& location) { _location = location; }
+	void setLocationExpression(Expression::Ptr&& location) { _location = std::move(location); }
+	
+	/**
+	 * Same as OfExpression::getLocationExpression
+	 * @note It is named as getRangeExpression to preserve backward compatibility
+	 */
+	const Expression::Ptr& getRangeExpression() const { return getLocationExpression(); }
+	void setRangeExpression(const Expression::Ptr& range) { setLocationExpression(range); }
+	void setRangeExpression(Expression::Ptr&& range) { setLocationExpression(std::move(range)); }
+
 	virtual TokenIt getFirstTokenIt() const override { return _forExpr->getFirstTokenIt(); }
-	virtual TokenIt getLastTokenIt() const override { return _iterable->getLastTokenIt(); }
+	virtual TokenIt getLastTokenIt() const override { return _location ? _location->getLastTokenIt() : _iterable->getLastTokenIt(); }
+
+private:
+	// Range and offset expression is stored in the same member _location, there cannot be offset and range at the same time
+	std::optional<TokenIt> _location_symbol; ///< Token holding "in" or "at"
+	Expression::Ptr _location; ///< Range expression ("in" <range>) or offset expression ("at" <offset>)
 };
 
 /**
@@ -1454,6 +1648,49 @@ protected:
 	TokenIt _symbolToken; ///< Token of the identifier
 };
 
+/**
+ * Class representing identifier wildcard expression. The only place where identifier wildcards
+ * can be used are N of (<rule_set>) expressions.
+ *
+ * For example:
+ * @code
+ * 2 of (rule_prefix_*)
+ *       ^^^^^^^^^^^^^
+ * @endcode
+ */
+class IdWildcardExpression : public Expression
+{
+public:
+	/**
+	 * Constructors
+	 */
+	IdWildcardExpression(TokenIt id, TokenIt wildcard) : _id(id), _wildcard(wildcard)
+	{
+	}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+
+	virtual std::string getText(const std::string& /*indent*/ = std::string{}) const override
+	{
+		return getId() + _wildcard->getString();
+	}
+
+	const std::string& getId() const { return _id->getString(); }
+
+	void setId(const std::string& id) { _id->setValue(id); }
+	void setId(std::string&& id) { _id->setValue(std::move(id)); }
+
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _wildcard; }
+
+protected:
+	TokenIt _id; ///< Token of the identifier wildcard
+	TokenIt _wildcard; ///< Token of the wildcard symbol
+};
+
 /**
  * Class representing access to the structure identifier. Structure identifier may only be imported module identifier,
  * or another attributes of the imported module structure.
@@ -1951,6 +2188,34 @@ public:
 	}
 };
 
+/**
+ * Class representing 'none' expression. Can be used in conjunction with for loops indicating that for loop
+ * needs to be evaluated false for all variables in the referenced set. This expression does not have a type.
+ *
+ * For example:
+ * @code
+ * none of them
+ * ^^^^
+ * @endcode
+ */
+class NoneExpression : public KeywordExpression
+{
+public:
+	NoneExpression(TokenIt t)
+		: KeywordExpression(t)
+	{
+	}
+	NoneExpression(const std::shared_ptr<TokenStream>& ts, TokenIt t)
+		: KeywordExpression(ts, t)
+	{
+	}
+
+	virtual VisitResult accept(Visitor* v) override
+	{
+		return v->visit(this);
+	}
+};
+
 /**
  * Class representing 'them' expression. Can be used in conjunction with string-based for loops referencing
  * all string from the strings section instead of specific set. This expression does not have a type.
