@@ -87,6 +87,14 @@ public:
 	TokenIt erase(TokenIt element);
 	TokenIt erase(TokenIt first, TokenIt last);
 
+
+	// Puts comment to the front.
+	void comment(const std::string& message, bool multiline = false, const std::string& indent = {}, bool linebreak = true);
+	// Puts comment to the back.
+	void commentBehind(const std::string& message, bool multiline = false, const std::string& indent = {}, bool linebreak = true);
+	// Puts comment before the insert_before parameter token.
+	void commentBeforeToken(const std::string& message, TokenIt insert_before, bool multiline = false, const std::string& indent = {}, bool linebreak = true);
+
 	// Steals all data from donor and append it at the end.
 	void moveAppend(TokenStream* donor);
 	// Steals all data from donor and append it at position before.
@@ -105,6 +113,7 @@ public:
 	const Token& front() const { return _tokens.front(); }
 	const Token& back() const { return _tokens.back(); }
 	const std::list<Token>& getTokens() const { return _tokens; }
+	std::pair<TokenIt, TokenIt> findBounds(TokenIt embedded, TokenType leftType, TokenType rightType) const;
 	/// @}
 
 	/// @name Iterators
@@ -142,6 +151,7 @@ public:
 	friend std::ostream& operator<<(std::ostream& os, TokenStream& ts) { return os << ts.getText(false); }
 	std::string getText(bool withIncludes = false, bool alignComments = true);
 	std::vector<std::string> getTokensAsText() const;
+	std::string getTokensAsString() const;
 	/// @}
 
 	/// @name New Line Characters
