@@ -220,11 +220,36 @@ public:
 		return _handleBinaryOperation(expr);
 	}
 
+	virtual VisitResult visit(IcontainsExpression* expr) override
+	{
+		return _handleBinaryOperation(expr);
+	}
+
+	virtual VisitResult visit(IendsWithExpression* expr) override
+	{
+		return _handleBinaryOperation(expr);
+	}
+
 	virtual VisitResult visit(IequalsExpression* expr) override
 	{
 		return _handleBinaryOperation(expr);
 	}
 
+	virtual VisitResult visit(IstartsWithExpression* expr) override
+	{
+		return _handleBinaryOperation(expr);
+	}
+
+	virtual VisitResult visit(EndsWithExpression* expr) override
+	{
+		return _handleBinaryOperation(expr);
+	}
+
+	virtual VisitResult visit(StartsWithExpression* expr) override
+	{
+		return _handleBinaryOperation(expr);
+	}
+
 	virtual VisitResult visit(PlusExpression* expr) override
 	{
 		return _handleBinaryOperation(expr);
@@ -570,6 +595,7 @@ public:
 		else
 			expr->setIterable(nullptr);
 
+		auto oldRangeExpr = expr->getRangeExpression();
 		if (auto rangeExpr = std::get_if<Expression::Ptr>(&rangeRet))
 		{
 			if (*rangeExpr)
@@ -579,7 +605,7 @@ public:
 			expr->setRangeExpression(nullptr);
 
 		// If any subnode needs to be deleted, we delete whole expr.
-		if (!expr->getVariable() || !expr->getIterable() || !expr->getRangeExpression())
+		if (!expr->getVariable() || !expr->getIterable() || (oldRangeExpr && !expr->getRangeExpression()))
 			return VisitAction::Delete;
 
 		return {};
