@@ -138,6 +138,13 @@ public:
 		return {};
 	}
 
+	virtual VisitResult visit(EndsWithExpression* expr) override
+	{
+		expr->getLeftOperand()->accept(this);
+		expr->getRightOperand()->accept(this);
+		return {};
+	}
+
 	virtual VisitResult visit(EqExpression* expr) override
 	{
 		expr->getLeftOperand()->accept(this);
@@ -166,6 +173,20 @@ public:
 		return {};
 	}
 
+	virtual VisitResult visit(IcontainsExpression* expr) override
+	{
+		expr->getLeftOperand()->accept(this);
+		expr->getRightOperand()->accept(this);
+		return {};
+	}
+
+	virtual VisitResult visit(IendsWithExpression* expr) override
+	{
+		expr->getLeftOperand()->accept(this);
+		expr->getRightOperand()->accept(this);
+		return {};
+	}
+
 	virtual VisitResult visit(IequalsExpression* expr) override
 	{
 		expr->getLeftOperand()->accept(this);
@@ -173,6 +194,13 @@ public:
 		return {};
 	}
 
+	virtual VisitResult visit(IstartsWithExpression* expr) override
+	{
+		expr->getLeftOperand()->accept(this);
+		expr->getRightOperand()->accept(this);
+		return {};
+	}
+
 	virtual VisitResult visit(PlusExpression* expr) override
 	{
 		expr->getLeftOperand()->accept(this);
@@ -229,6 +257,13 @@ public:
 		return {};
 	}
 
+	virtual VisitResult visit(StartsWithExpression* expr) override
+	{
+		expr->getLeftOperand()->accept(this);
+		expr->getRightOperand()->accept(this);
+		return {};
+	}
+
 	virtual VisitResult visit(ShiftLeftExpression* expr) override
 	{
 		expr->getLeftOperand()->accept(this);
