@@ -9,6 +9,7 @@
 #include <cctype>
 #include <sstream>
 #include <string>
+#include <cmath>
 
 namespace yaramod {
 
@@ -19,7 +20,7 @@ std::string unescapeString(std::string_view str);
 bool endsWith(const std::string& str, const std::string& withWhat);
 bool endsWith(const std::string& str, char withWhat);
 
-std::string trim(std::string str);
+std::string trim(std::string str, const std::string& toTrim = " \n\r\t\v");
 
 /**
  * Checks whether string starts with another string or character.
@@ -48,12 +49,44 @@ template <typename T>
 std::string numToStr(const T num, std::ios_base &(*format)(std::ios_base&) = std::dec, bool showbase = false, bool toUpper = false)
 {
 	std::ostringstream os;
+
+	// Set precision if num is floating point
+	if (std::is_floating_point<T>::value)
+		os.precision(std::numeric_limits<T>::digits10 - 1);
+
+	os << std::fixed;
 	if (toUpper)
 		os << std::uppercase;
 	if (showbase)
 		os << format << std::showbase << num;
 	else
 		os << format << num;
+
+	// Postprocess value if value is floating point
+	if (std::is_floating_point<T>::value)
+	{
+		std::string value = os.str();
+		auto comma = value.find('.');
+
+		// Generated string presentation do not have comma add .0 (xxxx)
+		if (comma == std::string::npos)
+		{
+			return value.append(".0");
+		}
+		// Generated string presentation have comma on last place (xxxx.)
+		if (comma == value.length() - 1)
+		{
+			return value.append("0");
+		}
+		// Generated string have format xxx.x
+		if (comma == value.length() - 2)
+		{
+			return value;
+		}
+		// Trim tailing zeros from generated float presentation (10.030000000 -> 10.03)
+		auto zero = value.find_last_not_of('0');
+		return {value.begin(), value.begin() + static_cast<long>(std::max(zero + 1, comma + 2))};
+	}
 	return os.str();
 }
 
