@@ -19,6 +19,7 @@ class StringInRangeExpression;
 class StringCountExpression;
 class StringOffsetExpression;
 class StringLengthExpression;
+class PercentualExpression;
 class NotExpression;
 class DefinedExpression;
 class UnaryMinusExpression;
@@ -32,7 +33,12 @@ class GeExpression;
 class EqExpression;
 class NeqExpression;
 class ContainsExpression;
+class IcontainsExpression;
 class MatchesExpression;
+class StartsWithExpression;
+class IstartsWithExpression;
+class EndsWithExpression;
+class IendsWithExpression;
 class IequalsExpression;
 class PlusExpression;
 class MinusExpression;
@@ -52,6 +58,7 @@ class IterableExpression;
 class SetExpression;
 class RangeExpression;
 class IdExpression;
+class IdWildcardExpression;
 class StructAccessExpression;
 class ArrayAccessExpression;
 class FunctionCallExpression;
@@ -63,6 +70,7 @@ class FilesizeExpression;
 class EntrypointExpression;
 class AllExpression;
 class AnyExpression;
+class NoneExpression;
 class ThemExpression;
 class ParenthesesExpression;
 class IntFunctionExpression;
@@ -85,6 +93,7 @@ public:
 	virtual VisitResult visit(StringCountExpression* expr) = 0;
 	virtual VisitResult visit(StringOffsetExpression* expr) = 0;
 	virtual VisitResult visit(StringLengthExpression* expr) = 0;
+	virtual VisitResult visit(PercentualExpression* expr) = 0;
 	virtual VisitResult visit(NotExpression* expr) = 0;
 	virtual VisitResult visit(DefinedExpression* expr) = 0;
 	virtual VisitResult visit(UnaryMinusExpression* expr) = 0;
@@ -98,7 +107,12 @@ public:
 	virtual VisitResult visit(EqExpression* expr) = 0;
 	virtual VisitResult visit(NeqExpression* expr) = 0;
 	virtual VisitResult visit(ContainsExpression* expr) = 0;
+	virtual VisitResult visit(IcontainsExpression* expr) = 0;
 	virtual VisitResult visit(MatchesExpression* expr) = 0;
+	virtual VisitResult visit(StartsWithExpression* expr) = 0;
+	virtual VisitResult visit(IstartsWithExpression* expr) = 0;
+	virtual VisitResult visit(EndsWithExpression* expr) = 0;
+	virtual VisitResult visit(IendsWithExpression* expr) = 0;
 	virtual VisitResult visit(IequalsExpression* expr) = 0;
 	virtual VisitResult visit(PlusExpression* expr) = 0;
 	virtual VisitResult visit(MinusExpression* expr) = 0;
@@ -118,6 +132,7 @@ public:
 	virtual VisitResult visit(SetExpression* expr) = 0;
 	virtual VisitResult visit(RangeExpression* expr) = 0;
 	virtual VisitResult visit(IdExpression* expr) = 0;
+	virtual VisitResult visit(IdWildcardExpression* expr) = 0;
 	virtual VisitResult visit(StructAccessExpression* expr) = 0;
 	virtual VisitResult visit(ArrayAccessExpression* expr) = 0;
 	virtual VisitResult visit(FunctionCallExpression* expr) = 0;
@@ -129,6 +144,7 @@ public:
 	virtual VisitResult visit(EntrypointExpression* expr) = 0;
 	virtual VisitResult visit(AllExpression* expr) = 0;
 	virtual VisitResult visit(AnyExpression* expr) = 0;
+	virtual VisitResult visit(NoneExpression* expr) = 0;
 	virtual VisitResult visit(ThemExpression* expr) = 0;
 	virtual VisitResult visit(ParenthesesExpression* expr) = 0;
 	virtual VisitResult visit(IntFunctionExpression* expr) = 0;
