@@ -539,6 +539,40 @@ YaraExpressionBuilder& YaraExpressionBuilder::matches(const YaraExpressionBuilde
 	return *this;
 }
 
+/**
+ * Applies operation icontains on two expressions.
+ *
+ * @param other The other expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder& YaraExpressionBuilder::icontains(const YaraExpressionBuilder& other)
+{
+	TokenIt token = _tokenStream->emplace_back(TokenType::ICONTAINS, "icontains");
+	_tokenStream->moveAppend(other.getTokenStream());
+
+	_expr = std::make_shared<IcontainsExpression>(std::move(_expr), token, other.get());
+	setType(Expression::Type::Bool);
+	return *this;
+}
+
+/**
+ * Applies operation iendswith on two expressions.
+ *
+ * @param other The other expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder& YaraExpressionBuilder::iendswith(const YaraExpressionBuilder& other)
+{
+	TokenIt token = _tokenStream->emplace_back(TokenType::IENDSWITH, "iendswith");
+	_tokenStream->moveAppend(other.getTokenStream());
+
+	_expr = std::make_shared<IendsWithExpression>(std::move(_expr), token, other.get());
+	setType(Expression::Type::Bool);
+	return *this;
+}
+
 /**
  * Applies operation iequals on two expressions.
  *
@@ -556,6 +590,57 @@ YaraExpressionBuilder& YaraExpressionBuilder::iequals(const YaraExpressionBuilde
 	return *this;
 }
 
+/**
+ * Applies operation istartswith on two expressions.
+ *
+ * @param other The other expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder& YaraExpressionBuilder::istartswith(const YaraExpressionBuilder& other)
+{
+	TokenIt token = _tokenStream->emplace_back(TokenType::ISTARTSWITH, "istartswith");
+	_tokenStream->moveAppend(other.getTokenStream());
+
+	_expr = std::make_shared<IstartsWithExpression>(std::move(_expr), token, other.get());
+	setType(Expression::Type::Bool);
+	return *this;
+}
+
+/**
+ * Applies operation startswith on two expressions.
+ *
+ * @param other The other expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder& YaraExpressionBuilder::startswith(const YaraExpressionBuilder& other)
+{
+	TokenIt token = _tokenStream->emplace_back(TokenType::STARTSWITH, "startswith");
+	_tokenStream->moveAppend(other.getTokenStream());
+
+	_expr = std::make_shared<StartsWithExpression>(std::move(_expr), token, other.get());
+	setType(Expression::Type::Bool);
+	return *this;
+}
+
+/**
+ * Applies operation endswith on two expressions.
+ *
+ * @param other The other expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder& YaraExpressionBuilder::endswith(const YaraExpressionBuilder& other)
+{
+	TokenIt token = _tokenStream->emplace_back(TokenType::ENDSWITH, "endswith");
+	_tokenStream->moveAppend(other.getTokenStream());
+
+	_expr = std::make_shared<EndsWithExpression>(std::move(_expr), token, other.get());
+	setType(Expression::Type::Bool);
+	return *this;
+}
+
 /**
  * Applies operation defined on the expression.
  *
@@ -1139,7 +1224,7 @@ YaraExpressionBuilder of(const YaraExpressionBuilder& ofExpr, const YaraExpressi
  *
  * @return Builder.
  */
-YaraExpressionBuilder of(const YaraExpressionBuilder& quantifier, const YaraExpressionBuilder& set, const YaraExpressionBuilder& range)
+YaraExpressionBuilder ofInRange(const YaraExpressionBuilder& quantifier, const YaraExpressionBuilder& set, const YaraExpressionBuilder& range)
 {
 	auto ts = std::make_shared<TokenStream>();
 	ts->moveAppend(quantifier.getTokenStream());
@@ -1153,6 +1238,29 @@ YaraExpressionBuilder of(const YaraExpressionBuilder& quantifier, const YaraExpr
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
 }
 
+/**
+ * Creates the expression with match of the specified set of strings at given offset.
+ *
+ * @param quantifier All / Any / None expression.
+ * @param set Set expression.
+ * @param offset Offset expression.
+ *
+ * @return Builder.
+ */
+YaraExpressionBuilder ofAt(const YaraExpressionBuilder& quantifier, const YaraExpressionBuilder& set, const YaraExpressionBuilder& offset)
+{
+	auto ts = std::make_shared<TokenStream>();
+	ts->moveAppend(quantifier.getTokenStream());
+	auto ofToken = ts->emplace_back(TokenType::OF, "of");
+	ts->moveAppend(set.getTokenStream());
+	auto atToken = ts->emplace_back(TokenType::OP_AT, "at");
+	ts->moveAppend(offset.getTokenStream());
+
+	auto expression = std::make_shared<OfExpression>(quantifier.get(), ofToken, set.get(), atToken, offset.get());
+
+	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
+}
+
 /**
  * Creates an iterable array of elements.
  *
