@@ -41,7 +41,12 @@ void addVisitorClasses(py::module& module)
 		.def("visit_NeqExpression", py::overload_cast<NeqExpression*>(&Visitor::visit))
 		.def("visit_ContainsExpression", py::overload_cast<ContainsExpression*>(&Visitor::visit))
 		.def("visit_MatchesExpression", py::overload_cast<MatchesExpression*>(&Visitor::visit))
+		.def("visit_IcontainsExpression", py::overload_cast<IcontainsExpression*>(&Visitor::visit))
+		.def("visit_IendsWithExpression", py::overload_cast<IendsWithExpression*>(&Visitor::visit))
 		.def("visit_IequalsExpression", py::overload_cast<IequalsExpression*>(&Visitor::visit))
+		.def("visit_IstartsWithExpression", py::overload_cast<IstartsWithExpression*>(&Visitor::visit))
+		.def("visit_StartsWithExpression", py::overload_cast<StartsWithExpression*>(&Visitor::visit))
+		.def("visit_EndsWithExpression", py::overload_cast<EndsWithExpression*>(&Visitor::visit))
 		.def("visit_PlusExpression", py::overload_cast<PlusExpression*>(&Visitor::visit))
 		.def("visit_MinusExpression", py::overload_cast<MinusExpression*>(&Visitor::visit))
 		.def("visit_MultiplyExpression", py::overload_cast<MultiplyExpression*>(&Visitor::visit))
@@ -103,7 +108,12 @@ void addVisitorClasses(py::module& module)
 		.def("visit_NeqExpression", py::overload_cast<NeqExpression*>(&ObservingVisitor::visit))
 		.def("visit_ContainsExpression", py::overload_cast<ContainsExpression*>(&ObservingVisitor::visit))
 		.def("visit_MatchesExpression", py::overload_cast<MatchesExpression*>(&ObservingVisitor::visit))
+		.def("visit_IcontainsExpression", py::overload_cast<IcontainsExpression*>(&ObservingVisitor::visit))
+		.def("visit_IendsWithExpression", py::overload_cast<IendsWithExpression*>(&ObservingVisitor::visit))
 		.def("visit_IequalsExpression", py::overload_cast<IequalsExpression*>(&ObservingVisitor::visit))
+		.def("visit_IstartsWithExpression", py::overload_cast<IstartsWithExpression*>(&ObservingVisitor::visit))
+		.def("visit_StartsWithExpression", py::overload_cast<StartsWithExpression*>(&ObservingVisitor::visit))
+		.def("visit_EndsWithExpression", py::overload_cast<EndsWithExpression*>(&ObservingVisitor::visit))
 		.def("visit_PlusExpression", py::overload_cast<PlusExpression*>(&ObservingVisitor::visit))
 		.def("visit_MinusExpression", py::overload_cast<MinusExpression*>(&ObservingVisitor::visit))
 		.def("visit_MultiplyExpression", py::overload_cast<MultiplyExpression*>(&ObservingVisitor::visit))
@@ -166,7 +176,12 @@ void addVisitorClasses(py::module& module)
 		.def("visit_NeqExpression", py::overload_cast<NeqExpression*>(&ModifyingVisitor::visit))
 		.def("visit_ContainsExpression", py::overload_cast<ContainsExpression*>(&ModifyingVisitor::visit))
 		.def("visit_MatchesExpression", py::overload_cast<MatchesExpression*>(&ModifyingVisitor::visit))
+		.def("visit_IcontainsExpression", py::overload_cast<IcontainsExpression*>(&ModifyingVisitor::visit))
+		.def("visit_IendsWithExpression", py::overload_cast<IendsWithExpression*>(&ModifyingVisitor::visit))
 		.def("visit_IequalsExpression", py::overload_cast<IequalsExpression*>(&ModifyingVisitor::visit))
+		.def("visit_IstartsWithExpression", py::overload_cast<IstartsWithExpression*>(&ModifyingVisitor::visit))
+		.def("visit_StartsWithExpression", py::overload_cast<StartsWithExpression*>(&ModifyingVisitor::visit))
+		.def("visit_EndsWithExpression", py::overload_cast<EndsWithExpression*>(&ModifyingVisitor::visit))
 		.def("visit_PlusExpression", py::overload_cast<PlusExpression*>(&ModifyingVisitor::visit))
 		.def("visit_MinusExpression", py::overload_cast<MinusExpression*>(&ModifyingVisitor::visit))
 		.def("visit_MultiplyExpression", py::overload_cast<MultiplyExpression*>(&ModifyingVisitor::visit))
@@ -221,7 +236,12 @@ void addVisitorClasses(py::module& module)
 		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, NeqExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
 		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ContainsExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
 		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, MatchesExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, IcontainsExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, IendsWithExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
 		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, IequalsExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, IstartsWithExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, StartsWithExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, EndsWithExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
 		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, PlusExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
 		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, MinusExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
 		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, MultiplyExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
