@@ -10,6 +10,7 @@
 #include <pybind11/pybind11.h>
 #include <pybind11/functional.h>
 #include <pybind11/stl.h>
+#include <pybind11/operators.h>
 
 #include <yaramod/builder/yara_expression_builder.h>
 #include <yaramod/builder/yara_hex_string_builder.h>
@@ -17,6 +18,7 @@
 #include <yaramod/types/expression.h>
 #include <yaramod/types/plain_string.h>
 #include <yaramod/types/token_type.h>
+#include <yaramod/types/token.h>
 #include <yaramod/yaramod.h>
 
 #include "yaramod_python.h"
@@ -169,7 +171,7 @@ void addEnums(py::module& module)
 		.value("Comma", TokenType::COMMA)
 		.value("Private", TokenType::PRIVATE)
 		.value("Global", TokenType::GLOBAL)
-		.value("None", TokenType::NONE)
+		.value("NoneToken", TokenType::NONE)
 		.value("Rule", TokenType::RULE)
 		.value("Variables", TokenType::VARIABLES)
 		.value("Strings", TokenType::STRINGS)
@@ -199,7 +201,12 @@ void addEnums(py::module& module)
 		.value("Filesize", TokenType::FILESIZE)
 		.value("Contains", TokenType::CONTAINS)
 		.value("Matches", TokenType::MATCHES)
+		.value("Icontains", TokenType::ICONTAINS)
+		.value("IendsWith", TokenType::IENDSWITH)
 		.value("Iequals", TokenType::IEQUALS)
+		.value("IstartsWith", TokenType::ISTARTSWITH)
+		.value("StartsWith", TokenType::STARTSWITH)
+		.value("EndsWith", TokenType::ENDSWITH)
 		.value("Slash", TokenType::SLASH)
 		.value("StringLiteral", TokenType::STRING_LITERAL)
 		.value("Integer", TokenType::INTEGER)
@@ -502,8 +509,12 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("symbol", &ReferenceSymbol::getSymbol);
 }
 
+PYBIND11_MAKE_OPAQUE(TokenIt);
+
 void addTokenStreamClass(py::module& module)
 {
+
+
 	py::class_<Token>(module, "Token")
 		.def(py::init<yaramod::TokenType, const Literal&>())
 		.def_property_readonly("text", [](Token& self) { return self.getText(); })
@@ -524,6 +535,24 @@ void addTokenStreamClass(py::module& module)
 		.def_property_readonly("symbol", &Token::getSymbol)
 		.def_property_readonly("literal_reference", &Token::getLiteralReference);
 
+	py::class_<TokenIt>(module, "TokenIt")
+		.def(py::init<>())
+		.def(pybind11::self == pybind11::self)
+		.def(pybind11::self != pybind11::self)
+		.def_property_readonly("value", [](const TokenIt& self) -> Token {
+			return *self;
+		})
+		.def("increment", [](TokenIt& self) { ++self; })
+		.def("decrement", [](TokenIt& self) { --self; })
+		.def("next", [](TokenIt& self) -> TokenIt {
+			TokenIt res = self;
+			return ++res;
+		})
+		.def("previous", [](TokenIt& self) -> TokenIt {
+			TokenIt res = self;
+			return --res;
+		});
+
 	py::class_<TokenStream, std::shared_ptr<TokenStream>>(module, "TokenStream")
 		.def(py::init<>())
 		.def_property_readonly("empty", &TokenStream::empty)
@@ -532,7 +561,18 @@ void addTokenStreamClass(py::module& module)
 		.def_property_readonly("back", &TokenStream::back)
 		.def_property_readonly("tokens", &TokenStream::getTokens)
 		.def_property_readonly("tokens_as_text", &TokenStream::getTokensAsText)
-		.def("comment_before_token", &TokenStream::commentBeforeToken, py::arg("message"), py::arg("insert_before"), py::arg("multiline") = false, py::arg("indent") = "", py::arg("linebreak") = true);
+		.def("comment_before_token", &TokenStream::commentBeforeToken, py::arg("message"), py::arg("insert_before"), py::arg("multiline") = false, py::arg("indent") = "", py::arg("linebreak") = true)
+		.def_property_readonly("begin", [](TokenStream& self) -> TokenIt { return self.begin(); })
+		.def_property_readonly("end", [](TokenStream& self)  -> TokenIt { return self.end(); })
+		.def("find", [](TokenStream& self, yaramod::TokenType type) -> TokenIt { return self.find(type); }, py::arg("type"))
+		.def("find_range", [](TokenStream& self, yaramod::TokenType type, TokenIt from, TokenIt to) -> TokenIt { return self.find(type, from, to); },
+			py::arg("type"), py::arg("from"), py::arg("to"))
+		.def("erase", [](TokenStream& self, TokenIt itr) -> TokenIt { return self.erase(itr); }, py::arg("itr"))
+		.def("erase_range", [](TokenStream& self, TokenIt first, TokenIt last) -> TokenIt { return self.erase(first, last); },
+			py::arg("first"), py::arg("last"))
+		.def("insert", [](TokenStream& self, TokenIt before, yaramod::TokenType type, const Literal& literal) -> TokenIt {
+			return self.insert(before, type, literal);
+		}, py::arg("before"), py::arg("type"), py::arg("literal"));
 }
 
 void addExpressionClasses(py::module& module)
@@ -623,7 +663,12 @@ void addExpressionClasses(py::module& module)
 	binaryOpClass<NeqExpression>(module, "NeqExpression");
 	binaryOpClass<ContainsExpression>(module, "ContainsExpression");
 	binaryOpClass<MatchesExpression>(module, "MatchesExpression");
+	binaryOpClass<IcontainsExpression>(module, "IcontainsExpression");
+	binaryOpClass<IendsWithExpression>(module, "IendsWithExpression");
 	binaryOpClass<IequalsExpression>(module, "IequalsExpression");
+	binaryOpClass<IstartsWithExpression>(module, "IstartsWithExpression");
+	binaryOpClass<StartsWithExpression>(module, "StartsWithExpression");
+	binaryOpClass<EndsWithExpression>(module, "EndsWithExpression");
 	binaryOpClass<PlusExpression>(module, "PlusExpression");
 	binaryOpClass<MinusExpression>(module, "MinusExpression");
 	binaryOpClass<MultiplyExpression>(module, "MultiplyExpression");
@@ -660,7 +705,10 @@ void addExpressionClasses(py::module& module)
 	exprClass<OfExpression, ForExpression>(module, "OfExpression")
 		.def_property("range",
 				&OfExpression::getRangeExpression,
-				py::overload_cast<const Expression::Ptr&>(&OfExpression::setRangeExpression));
+				py::overload_cast<const Expression::Ptr&>(&OfExpression::setRangeExpression))
+		.def_property("location",
+				&OfExpression::getLocationExpression,
+				py::overload_cast<const Expression::Ptr&>(&OfExpression::setLocationExpression));
 
 	exprClass<IterableExpression>(module, "IterableExpression")
 		.def_property("elements",
@@ -847,7 +895,12 @@ void addBuilderClasses(py::module& module)
 		.def("comment_behind", &YaraExpressionBuilder::commentBehind, py::arg("message"), py::arg("multiline") = false, py::arg("indent") = "", py::arg("linebreak") = true)
 		.def("contains", &YaraExpressionBuilder::contains)
 		.def("matches", &YaraExpressionBuilder::matches)
+		.def("icontains", &YaraExpressionBuilder::icontains)
+		.def("iendswith", &YaraExpressionBuilder::iendswith)
 		.def("iequals", &YaraExpressionBuilder::iequals)
+		.def("istartswith", &YaraExpressionBuilder::istartswith)
+		.def("startswith", &YaraExpressionBuilder::startswith)
+		.def("endswith", &YaraExpressionBuilder::endswith)
 		.def("defined", &YaraExpressionBuilder::defined)
 		.def("percent", &YaraExpressionBuilder::percent)
 		.def("read_int8", &YaraExpressionBuilder::readInt8)
@@ -899,7 +952,8 @@ void addBuilderClasses(py::module& module)
 			const YaraExpressionBuilder&
 		>(&forLoop));
 	module.def("of", py::overload_cast<const YaraExpressionBuilder&, const YaraExpressionBuilder&>(&of));
-	module.def("of", py::overload_cast<const YaraExpressionBuilder&, const YaraExpressionBuilder&, const YaraExpressionBuilder&>(&of));
+	module.def("of_in_range", py::overload_cast<const YaraExpressionBuilder&, const YaraExpressionBuilder&, const YaraExpressionBuilder&>(&ofInRange));
+	module.def("of_at", py::overload_cast<const YaraExpressionBuilder&, const YaraExpressionBuilder&, const YaraExpressionBuilder&>(&ofAt));
 
 	module.def("iterable", &iterable);
 
@@ -935,6 +989,9 @@ void addBuilderClasses(py::module& module)
 	module.def("wildcard", &wildcard);
 	module.def("wildcard_low", &wildcardLow);
 	module.def("wildcard_high", &wildcardHigh);
+	module.def("not_byte", &notByte);
+	module.def("not_wildcard_low", &notWildcardLow);
+	module.def("not_wildcard_high", &notWildcardHigh);
 
 	module.def("jump_varying", &jumpVarying);
 	module.def("jump_fixed", &jumpFixed);
