@@ -77,11 +77,9 @@ void YaraFile::initializeVTSymbols()
 		std::make_shared<ValueSymbol>("file_type", Expression::Type::String),
 		std::make_shared<ValueSymbol>("file_name", Expression::Type::String),
 		// VirusTotal specific global variables of antiviruses
-		std::make_shared<ValueSymbol>("a_squared", Expression::Type::String),
 		std::make_shared<ValueSymbol>("acronis", Expression::Type::String),
 		std::make_shared<ValueSymbol>("ad_aware", Expression::Type::String),
 		std::make_shared<ValueSymbol>("aegislab", Expression::Type::String),
-		std::make_shared<ValueSymbol>("agnitum", Expression::Type::String),
 		std::make_shared<ValueSymbol>("ahnlab", Expression::Type::String),
 		std::make_shared<ValueSymbol>("ahnlab_v3", Expression::Type::String),
 		std::make_shared<ValueSymbol>("alibaba", Expression::Type::String),
@@ -91,16 +89,17 @@ void YaraFile::initializeVTSymbols()
 		std::make_shared<ValueSymbol>("antiy_avl", Expression::Type::String),
 		std::make_shared<ValueSymbol>("apex", Expression::Type::String),
 		std::make_shared<ValueSymbol>("arcabit", Expression::Type::String),
-		std::make_shared<ValueSymbol>("authentium", Expression::Type::String),
 		std::make_shared<ValueSymbol>("avast", Expression::Type::String),
 		std::make_shared<ValueSymbol>("avast_mobile", Expression::Type::String),
 		std::make_shared<ValueSymbol>("avg", Expression::Type::String),
 		std::make_shared<ValueSymbol>("avira", Expression::Type::String),
 		std::make_shared<ValueSymbol>("avware", Expression::Type::String),
+		std::make_shared<ValueSymbol>("babable", Expression::Type::String),
 		std::make_shared<ValueSymbol>("baidu", Expression::Type::String),
 		std::make_shared<ValueSymbol>("bitdefender", Expression::Type::String),
+		std::make_shared<ValueSymbol>("bitdefenderfalx", Expression::Type::String),
+		std::make_shared<ValueSymbol>("bitdefendertheta", Expression::Type::String),
 		std::make_shared<ValueSymbol>("bkav", Expression::Type::String),
-		std::make_shared<ValueSymbol>("bytehero", Expression::Type::String),
 		std::make_shared<ValueSymbol>("cat_quickheal", Expression::Type::String),
 		std::make_shared<ValueSymbol>("clamav", Expression::Type::String),
 		std::make_shared<ValueSymbol>("cmc", Expression::Type::String),
@@ -113,15 +112,18 @@ void YaraFile::initializeVTSymbols()
 		std::make_shared<ValueSymbol>("cyren", Expression::Type::String),
 		std::make_shared<ValueSymbol>("drweb", Expression::Type::String),
 		std::make_shared<ValueSymbol>("egambit", Expression::Type::String),
+		std::make_shared<ValueSymbol>("elastic", Expression::Type::String),
 		std::make_shared<ValueSymbol>("emsisoft", Expression::Type::String),
 		std::make_shared<ValueSymbol>("endgame", Expression::Type::String),
-		std::make_shared<ValueSymbol>("esafe", Expression::Type::String),
 		std::make_shared<ValueSymbol>("escan", Expression::Type::String),
 		std::make_shared<ValueSymbol>("eset_nod32", Expression::Type::String),
 		std::make_shared<ValueSymbol>("f_prot", Expression::Type::String),
 		std::make_shared<ValueSymbol>("f_secure", Expression::Type::String),
+		std::make_shared<ValueSymbol>("fireeye", Expression::Type::String),
 		std::make_shared<ValueSymbol>("fortinet", Expression::Type::String),
 		std::make_shared<ValueSymbol>("gdata", Expression::Type::String),
+		std::make_shared<ValueSymbol>("google", Expression::Type::String),
+		std::make_shared<ValueSymbol>("gridinsoft", Expression::Type::String),
 		std::make_shared<ValueSymbol>("ikarus", Expression::Type::String),
 		std::make_shared<ValueSymbol>("invincea", Expression::Type::String),
 		std::make_shared<ValueSymbol>("jiangmin", Expression::Type::String),
@@ -129,6 +131,7 @@ void YaraFile::initializeVTSymbols()
 		std::make_shared<ValueSymbol>("k7gw", Expression::Type::String),
 		std::make_shared<ValueSymbol>("kaspersky", Expression::Type::String),
 		std::make_shared<ValueSymbol>("kingsoft", Expression::Type::String),
+		std::make_shared<ValueSymbol>("lionic", Expression::Type::String),
 		std::make_shared<ValueSymbol>("malwarebytes", Expression::Type::String),
 		std::make_shared<ValueSymbol>("max", Expression::Type::String),
 		std::make_shared<ValueSymbol>("maxsecure", Expression::Type::String),
@@ -138,12 +141,9 @@ void YaraFile::initializeVTSymbols()
 		std::make_shared<ValueSymbol>("microworld_escan", Expression::Type::String),
 		std::make_shared<ValueSymbol>("nano_antivirus", Expression::Type::String),
 		std::make_shared<ValueSymbol>("nod32", Expression::Type::String),
-		std::make_shared<ValueSymbol>("norman", Expression::Type::String),
 		std::make_shared<ValueSymbol>("nprotect", Expression::Type::String),
 		std::make_shared<ValueSymbol>("paloalto", Expression::Type::String),
 		std::make_shared<ValueSymbol>("panda", Expression::Type::String),
-		std::make_shared<ValueSymbol>("pctools", Expression::Type::String),
-		std::make_shared<ValueSymbol>("prevx", Expression::Type::String),
 		std::make_shared<ValueSymbol>("prevx1", Expression::Type::String),
 		std::make_shared<ValueSymbol>("qihoo_360", Expression::Type::String),
 		std::make_shared<ValueSymbol>("rising", Expression::Type::String),
@@ -158,6 +158,7 @@ void YaraFile::initializeVTSymbols()
 		std::make_shared<ValueSymbol>("tencent", Expression::Type::String),
 		std::make_shared<ValueSymbol>("thehacker", Expression::Type::String),
 		std::make_shared<ValueSymbol>("totaldefense", Expression::Type::String),
+		std::make_shared<ValueSymbol>("trapmine", Expression::Type::String),
 		std::make_shared<ValueSymbol>("trendmicro", Expression::Type::String),
 		std::make_shared<ValueSymbol>("trendmicro_housecall", Expression::Type::String),
 		std::make_shared<ValueSymbol>("trustlook", Expression::Type::String),
