@@ -6,6 +6,7 @@
 
 #include <algorithm>
 #include <cctype>
+#include <cstdint>
 #include <iomanip>
 #include <locale>
 #include <optional>
@@ -67,6 +68,9 @@ std::string escapeString(const std::string& str)
 			case '\n':
 				result += "\\n";
 				break;
+			case '\r':
+				result += "\\r";
+				break;
 			case '\t':
 				result += "\\t";
 				break;
