@@ -241,6 +241,54 @@ RuleWithMetasWorks) {
 )", yaraFile->getTextFormatted());
 }
 
+TEST_F(BuilderTests,
+RuleWithVariablesWorks) {
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+		.withName("rule_with_variables")
+		.withStringVariable("string_var", "string value")
+		.withIntVariable("int_var", 42)
+		.withDoubleVariable("double_var", 2.6)
+		.withBoolVariable("bool_var", false)
+		.withStructVariable("struct_var", "time")
+		.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+		.withModule("time")
+		.withRule(std::move(rule))
+		.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(import "time"
+
+rule rule_with_variables {
+	variables:
+		string_var = "string value"
+		int_var = 42
+		double_var = 2.6
+		bool_var = false
+		struct_var = time
+	condition:
+		true
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(import "time"
+
+rule rule_with_variables
+{
+	variables:
+		string_var = "string value"
+		int_var = 42
+		double_var = 2.6
+		bool_var = false
+		struct_var = time
+	condition:
+		true
+}
+)", yaraFile->getTextFormatted());
+}
+
 TEST_F(BuilderTests,
 RuleWithEmptyStringMetaValueWorks) {
 	YaraRuleBuilder newRule;
@@ -515,6 +563,8 @@ rule rule_2 : Tag2
 TEST_F(BuilderTests,
 RuleWithCustomConditionWorks) {
 	auto cond = matchAt("$1", entrypoint()).get();
+	EXPECT_EQ("$1", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("entrypoint", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -548,7 +598,14 @@ RuleWithCustomConditionWorks) {
 
 TEST_F(BuilderTests,
 RuleWithConditionWithSymbolsWorks) {
-	auto cond = forLoop(any(), "i", set({intVal(1), intVal(2), intVal(3)}), matchAt("$1", paren(entrypoint() + id("i")))).get();
+	auto cond = forLoop(any(), "n", set({intVal(1), intVal(2), intVal(3)}), matchAt("$1", paren(entrypoint() + id("i")))).get();
+	EXPECT_EQ("for", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
+	auto forExp = std::static_pointer_cast<ForArrayExpression>(cond);
+	EXPECT_EQ(forExp->getId(), "n");
+	forExp->setId("i");
+	EXPECT_EQ(forExp->getId(), "i");
+	EXPECT_EQ(cond->getText(), "for any i in (1, 2, 3) : ( $1 at (entrypoint + i) )");
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -583,6 +640,8 @@ RuleWithConditionWithSymbolsWorks) {
 TEST_F(BuilderTests,
 RuleWithHexStringWorks) {
 	auto cond = stringRef("$1").get();
+	EXPECT_EQ("$1", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("$1", cond->getLastTokenIt()->getPureText());
 
 	auto alt1 = YaraHexStringBuilder().add(0xBB).add(0xCC);
 	auto alt2 = YaraHexStringBuilder().add(0xDD, 0xEE);
@@ -591,7 +650,7 @@ RuleWithHexStringWorks) {
 
 	YaraHexStringBuilder newHexStr;
 	auto hexStr = newHexStr
-		.add(0x11, 0x22, wildcard(), wildcardHigh(0xA), wildcardLow(0xB))
+		.add(0x11, 0x22, wildcard(), wildcardHigh(0xA), wildcardLow(0xB), notByte(0x31), notWildcardHigh(0x3), notWildcardLow(0x4))
 		.add(jumpVarying(), jumpFixed(5), jumpVaryingRange(3), jumpRange(3, 5))
 		.add(alt(alt(alt1, alt2), alt3, alt4))
 		.add(0x99)
@@ -614,7 +673,7 @@ RuleWithHexStringWorks) {
 	ASSERT_NE(nullptr, yaraFile);
 	EXPECT_EQ(R"(rule rule_with_hex_string {
 	strings:
-		$1 = { 11 22 ?? ?A B? [-] [5] [3-] [3-5] ( ( BB CC | DD EE ) | FF F1 | FE ED DC ) 99 }
+		$1 = { 11 22 ?? ?A B? ~31 ~?3 ~4? [-] [5] [3-] [3-5] ( ( BB CC | DD EE ) | FF F1 | FE ED DC ) 99 }
 	condition:
 		$1
 })", yaraFile->getText());
@@ -622,16 +681,57 @@ RuleWithHexStringWorks) {
 	EXPECT_EQ(R"(rule rule_with_hex_string
 {
 	strings:
-		$1 = { 11 22 ?? ?A B? [-] [5] [3-] [3-5] ( ( BB CC | DD EE ) | FF F1 | FE ED DC ) 99 }
+		$1 = { 11 22 ?? ?A B? ~31 ~?3 ~4? [-] [5] [3-] [3-5] ( ( BB CC | DD EE ) | FF F1 | FE ED DC ) 99 }
 	condition:
 		$1
 }
 )", yaraFile->getTextFormatted());
 }
 
+TEST_F(BuilderTests,
+RuleWithDictForConditionWorks) {
+	auto cond = forLoop(any(), "k", "v", id("pe").access("os_version"), id("v") == intVal(3)).get();
+	EXPECT_EQ("for", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+		.withName("rule_with_dict_for_condition")
+		.withPlainString("$1", "Hello World!")
+		.withPlainString("$2", "Ahoj Svet!")
+		.withCondition(cond)
+		.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+		.withRule(std::move(rule))
+		.get(false, &driver);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule rule_with_dict_for_condition {
+	strings:
+		$1 = "Hello World!"
+		$2 = "Ahoj Svet!"
+	condition:
+		for any k, v in pe.os_version : ( v == 3 )
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule rule_with_dict_for_condition
+{
+	strings:
+		$1 = "Hello World!"
+		$2 = "Ahoj Svet!"
+	condition:
+		for any k, v in pe.os_version : ( v == 3 )
+}
+)", yaraFile->getTextFormatted());
+}
+
 TEST_F(BuilderTests,
 RuleWithStringForConditionWorks) {
 	auto cond = forLoop(any(), set({stringRef("$1"), stringRef("$2")}), matchAt("$", entrypoint())).get();
+	EXPECT_EQ("for", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -669,6 +769,8 @@ RuleWithStringForConditionWorks) {
 TEST_F(BuilderTests,
 RuleWithOfWorks) {
 	auto cond = of(all(), them()).get();
+	EXPECT_EQ("all", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("them", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -706,6 +808,8 @@ RuleWithOfWorks) {
 TEST_F(BuilderTests,
 RuleWithRangeWorks) {
 	auto cond = matchInRange("$1", range(intVal(0), filesize())).get();
+	EXPECT_EQ("$1", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -737,9 +841,50 @@ RuleWithRangeWorks) {
 )", yaraFile->getTextFormatted());
 }
 
+TEST_F(BuilderTests,
+RuleWithOfAtWorks) {
+	auto cond = ofAt(any(), them(), intVal(10)).get();
+	EXPECT_EQ("any", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("10", cond->getLastTokenIt()->getPureText());
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+		.withName("rule_with_of_at")
+		.withPlainString("$1", "Hello World!")
+		.withPlainString("$2", "Ahoj Svet!")
+		.withCondition(cond)
+		.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+		.withRule(std::move(rule))
+		.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule rule_with_of_at {
+	strings:
+		$1 = "Hello World!"
+		$2 = "Ahoj Svet!"
+	condition:
+		any of them at 10
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule rule_with_of_at
+{
+	strings:
+		$1 = "Hello World!"
+		$2 = "Ahoj Svet!"
+	condition:
+		any of them at 10
+}
+)", yaraFile->getTextFormatted());
+}
+
 TEST_F(BuilderTests,
 RuleWithStructureWorks) {
 	auto cond = (id("pe").access("number_of_sections") > intVal(1)).get();
+	EXPECT_EQ("pe", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("1", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -773,7 +918,9 @@ rule rule_with_range
 
 TEST_F(BuilderTests,
 RuleWithConditionWithOnelineComment) {
-	auto cond = (id("pe").comment("Number of sections needs to exceed 1", false).access("number_of_sections") > intVal(1)).get();
+	auto cond = (id("pe").access("number_of_sections") > intVal(1)).comment("Number of sections needs to exceed 1", false).get();
+	EXPECT_EQ("pe", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("1", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -808,7 +955,7 @@ rule rule_with_range
 
 TEST_F(BuilderTests,
 RuleWithConditionWithMultilineComment) {
-	auto cond = (id("pe").comment("Number of sections needs to exceed 1,\n\t\tbecause one is simply not enough.", true).access("number_of_sections") > intVal(1)).get();
+	auto cond = (id("pe").access("number_of_sections") > intVal(1)).comment("Number of sections needs to exceed 1,\n\t\tbecause one is simply not enough.", true).get();
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -846,6 +993,8 @@ TEST_F(BuilderTests,
 RuleWithArrayAndStructureWorks) {
 	auto cond = id("pe").access("sections")[intVal(0)].access("name").contains(stringVal("text"))
 		.get();
+	EXPECT_EQ("pe", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("text", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -881,6 +1030,8 @@ TEST_F(BuilderTests,
 RuleWithFunctionCallWorks) {
 	auto cond = id("pe").access("exports")(stringVal("ExitProcess"))
 		.get();
+	EXPECT_EQ("pe", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -916,6 +1067,8 @@ TEST_F(BuilderTests,
 RuleWithIntFunctionWorks) {
 	auto cond = (intVal(0).readUInt16(IntFunctionEndianness::Little) == hexIntVal(0x5A4D))
 		.get();
+	EXPECT_EQ("uint16", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("0x5a4d", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -946,6 +1099,8 @@ TEST_F(BuilderTests,
 RuleWithArithmeticOperationsWorks) {
 	auto cond = (paren(entrypoint() + intVal(100) * intVal(3)) < paren(filesize() - intVal(100) / intVal(2)))
 		.get();
+	EXPECT_EQ("(", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1006,6 +1161,8 @@ TEST_F(BuilderTests,
 RuleWithBitwiseOperationsWorks) {
 	auto cond = (id("pe").access("characteristics") & paren(id("pe").access("DLL") | id("pe").access("RELOCS_STRIPPED")))
 		.get();
+	EXPECT_EQ("pe", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1041,6 +1198,8 @@ TEST_F(BuilderTests,
 RuleWithLogicOperationsWorks) {
 	auto cond = (id("pe").access("is_32bit")() && paren((id("pe").access("is_dll")() || paren(id("pe").access("number_of_sections") > intVal(3)))))
 		.get();
+	EXPECT_EQ("pe", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1080,6 +1239,8 @@ TEST_F(BuilderTests,
 RuleWithIntMultpliersWorks) {
 	auto cond = (intVal(100, IntMultiplier::Kilobytes) <= filesize() && filesize() <= intVal(1, IntMultiplier::Megabytes))
 		.get();
+	EXPECT_EQ("100KB", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("1MB", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1111,6 +1272,8 @@ TEST_F(BuilderTests,
 RuleWithStringOperatorsWorks) {
 	auto cond = (matchCount("$1") > intVal(0) && matchLength("$1") > intVal(1) && matchOffset("$1") > intVal(100))
 		.get();
+	EXPECT_EQ("#1", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("100", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1182,6 +1345,8 @@ TEST_F(BuilderTests,
 RuleWithRegexpInConditionWorks) {
 	auto cond = (id("pe").access("sections")[intVal(0)].access("name").matches(regexp(R"(\.(text|data))", "i")))
 		.get();
+	EXPECT_EQ("pe", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ("i", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1217,6 +1382,8 @@ TEST_F(BuilderTests,
 RuleWithConjunctionInConditionWorks) {
 	std::vector<YaraExpressionBuilder> terms = { stringRef("$1"), paren(matchOffset("$1") < intVal(100)), paren(entrypoint() == intVal(100)) };
 	auto cond = conjunction(terms).get();
+	EXPECT_EQ("$1", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1254,6 +1421,8 @@ TEST_F(BuilderTests,
 RuleWithDisjunctionInConditionWorks) {
 	std::vector<YaraExpressionBuilder> terms = { stringRef("$1"), stringRef("$2"), paren(entrypoint() == intVal(100)) };
 	auto cond = disjunction(terms).get();
+	EXPECT_EQ("$1", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
@@ -1407,12 +1576,9 @@ RuleWithCommentedConjunctionInConditionWorks) {
 	strings:
 		$1 = "Hello"
 	condition:
-		/* comment1 */
-		$1 and
-		/* comment2 */
-		(@1 < 100) and
-		/* comment3 */
-		(entrypoint == 100)
+		$1 and              // comment1
+		(@1 < 100) and      // comment2
+		(entrypoint == 100) // comment3
 }
 )", yaraFile->getTextFormatted());
 }
@@ -1458,12 +1624,9 @@ RuleWithCommentedDisjunctionInConditionWorks) {
 	condition:
 		true and
 		(
-			/* Hello must be present */
-			$1 or
-			/* World must be present */
-			$2 or
-			/* entrypoint is 100 */
-			(entrypoint == 100)
+			$1 or               // Hello must be present
+			$2 or               // World must be present
+			(entrypoint == 100) // entrypoint is 100
 		)
 }
 )", yaraFile->getTextFormatted());
@@ -1517,13 +1680,13 @@ RuleWithParenthesesWithLinebreaksInConditionWorks) {
 
 TEST_F(BuilderTests,
 RuleWithEscapedSequencesWorks) {
-	auto cond = (id("pe").access("rich_signature").access("clear_data") == stringVal("DanS\"\t\n\\\x01\xff")).get();
+	auto cond = (id("pe").access("rich_signature").access("clear_data") == stringVal("DanS\"\t\r\n\\\x01\xff")).get();
 
 	YaraRuleBuilder newRule;
 	auto rule = newRule
 		.withName("rule_with_double_quotes")
-		.withStringMeta("str_meta", "Double \"\t\n\\\x01\xff quotes")
-		.withPlainString("$str", "Double \"\t\n\\\x01\xff quotes")
+		.withStringMeta("str_meta", "Double \"\t\r\n\\\x01\xff quotes")
+		.withPlainString("$str", "Double \"\t\r\n\\\x01\xff quotes")
 		.withCondition(cond)
 		.get();
 
@@ -1539,11 +1702,11 @@ RuleWithEscapedSequencesWorks) {
 
 rule rule_with_double_quotes {
 	meta:
-		str_meta = "Double \"\t\n\\\x01\xff quotes"
+		str_meta = "Double \"\t\r\n\\\x01\xff quotes"
 	strings:
-		$str = "Double \"\t\n\\\x01\xff quotes"
+		$str = "Double \"\t\r\n\\\x01\xff quotes"
 	condition:
-		pe.rich_signature.clear_data == "DanS\"\t\n\\\x01\xff"
+		pe.rich_signature.clear_data == "DanS\"\t\r\n\\\x01\xff"
 })";
 	EXPECT_EQ(expected, yaraFile->getText());
 
@@ -1552,16 +1715,89 @@ rule rule_with_double_quotes {
 rule rule_with_double_quotes
 {
 	meta:
-		str_meta = "Double \"\t\n\\\x01\xff quotes"
+		str_meta = "Double \"\t\r\n\\\x01\xff quotes"
 	strings:
-		$str = "Double \"\t\n\\\x01\xff quotes"
+		$str = "Double \"\t\r\n\\\x01\xff quotes"
 	condition:
-		pe.rich_signature.clear_data == "DanS\"\t\n\\\x01\xff"
+		pe.rich_signature.clear_data == "DanS\"\t\r\n\\\x01\xff"
 }
 )";
 	EXPECT_EQ(expected_with_newline, yaraFile->getTextFormatted());
 }
 
+TEST_F(BuilderTests,
+RuleWithNoneOfThemExpression) {
+	auto cond = of(none(), them()).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+		.withName("rule_with_none_of_them_expression")
+		.withPlainString("$1", "Hello").ascii()
+		.withPlainString("$2", "World").ascii().wide().private_()
+		.withCondition(cond)
+		.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+		.withRule(std::move(rule))
+		.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule rule_with_none_of_them_expression {
+	strings:
+		$1 = "Hello"
+		$2 = "World" ascii wide private
+	condition:
+		none of them
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule rule_with_none_of_them_expression
+{
+	strings:
+		$1 = "Hello"
+		$2 = "World" ascii wide private
+	condition:
+		none of them
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+RuleWithNoneOfThemInRangeWorks) {
+	auto cond = ofInRange(none(), them(), range(intVal(0), filesize())).get();
+	EXPECT_EQ("none", cond->getFirstTokenIt()->getPureText());
+	EXPECT_EQ(")", cond->getLastTokenIt()->getPureText());
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+		.withName("rule_with_range")
+		.withPlainString("$1", "Hello World!")
+		.withCondition(cond)
+		.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+		.withRule(std::move(rule))
+		.get(true, &driver);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule rule_with_range {
+	strings:
+		$1 = "Hello World!"
+	condition:
+		none of them in (0 .. filesize)
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule rule_with_range
+{
+	strings:
+		$1 = "Hello World!"
+	condition:
+		none of them in (0 .. filesize)
+}
+)", yaraFile->getTextFormatted());
+}
+
 TEST_F(BuilderTests,
 RuleWithStringsWithDifferentKindsOfModifiers) {
 	auto cond = of(all(), them()).get();
@@ -1574,11 +1810,15 @@ RuleWithStringsWithDifferentKindsOfModifiers) {
 		.withPlainString("$3", "World").ascii().wide().private_()
 		.withHexString("$4", YaraHexStringBuilder{std::vector<std::uint8_t>{0x01, 0x02, 0x03, 0x04}}.get())
 		.withRegexp("$5", "abc", "i").wide().nocase()
-		.withPlainString("$6", "Bye").nocase().xor_()
+		.withPlainString("$6", "Bye").wide().xor_()
 		.withPlainString("$7", "Bye").fullword().xor_()
 		.withPlainString("$8", "Bye").xor_(12)
 		.withPlainString("$9", "Bye").xor_(1, 255)
 		.withPlainString("$10", "Bye").xor_(128).xor_(1, 255)
+		.withPlainString("$11", "Bye").base64()
+		.withPlainString("$12", "Bye").base64("!@#$%^&*(){}[].,|ABCDEFGHIJ\x09LMNOPQRSTUVWXYZabcdefghijklmnopqrstu")
+		.withPlainString("$13", "Bye").base64wide()
+		.withPlainString("$14", "Bye").base64wide("!@#$%^&*(){}[].,|ABCDEFGHIJ\x09LMNOPQRSTUVWXYZabcdefghijklmnopqrstu")
 		.withCondition(cond)
 		.get();
 
@@ -1595,11 +1835,15 @@ RuleWithStringsWithDifferentKindsOfModifiers) {
 		$3 = "World" ascii wide private
 		$4 = { 01 02 03 04 }
 		$5 = /abc/i wide nocase
-		$6 = "Bye" nocase xor
+		$6 = "Bye" wide xor
 		$7 = "Bye" fullword xor
 		$8 = "Bye" xor(12)
 		$9 = "Bye" xor(1-255)
 		$10 = "Bye" xor(128)
+		$11 = "Bye" base64
+		$12 = "Bye" base64("!@#$%^&*(){}[].,|ABCDEFGHIJ\tLMNOPQRSTUVWXYZabcdefghijklmnopqrstu")
+		$13 = "Bye" base64wide
+		$14 = "Bye" base64wide("!@#$%^&*(){}[].,|ABCDEFGHIJ\tLMNOPQRSTUVWXYZabcdefghijklmnopqrstu")
 	condition:
 		all of them
 })", yaraFile->getText());
@@ -1612,11 +1856,15 @@ RuleWithStringsWithDifferentKindsOfModifiers) {
 		$3 = "World" ascii wide private
 		$4 = { 01 02 03 04 }
 		$5 = /abc/i wide nocase
-		$6 = "Bye" nocase xor
+		$6 = "Bye" wide xor
 		$7 = "Bye" fullword xor
 		$8 = "Bye" xor(12)
 		$9 = "Bye" xor(1-255)
 		$10 = "Bye" xor(128)
+		$11 = "Bye" base64
+		$12 = "Bye" base64("!@#$%^&*(){}[].,|ABCDEFGHIJ\tLMNOPQRSTUVWXYZabcdefghijklmnopqrstu")
+		$13 = "Bye" base64wide
+		$14 = "Bye" base64wide("!@#$%^&*(){}[].,|ABCDEFGHIJ\tLMNOPQRSTUVWXYZabcdefghijklmnopqrstu")
 	condition:
 		all of them
 }
@@ -1756,5 +2004,380 @@ ConjunctionWithSingleTerm) {
 )", yaraFile->getTextFormatted());
 }
 
+TEST_F(BuilderTests,
+DefinedTerm) {
+	auto cond = boolVal(false).defined().get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("defined_rule")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule defined_rule {
+	condition:
+		defined false
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule defined_rule
+{
+	condition:
+		defined false
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+PercentageOfStringSet) {
+	auto cond = of(intVal(50).percent(), them()).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+		.withName("rule_with_percentage")
+		.withPlainString("$1", "Hello World!")
+		.withPlainString("$2", "Ahoj Svet!")
+		.withCondition(cond)
+		.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+		.withRule(std::move(rule))
+		.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule rule_with_percentage {
+	strings:
+		$1 = "Hello World!"
+		$2 = "Ahoj Svet!"
+	condition:
+		50% of them
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule rule_with_percentage
+{
+	strings:
+		$1 = "Hello World!"
+		$2 = "Ahoj Svet!"
+	condition:
+		50% of them
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+FloatValueWorks) {
+	auto cond = doubleVal(0.0).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+		.withName("float_value_works")
+		.withCondition(cond)
+		.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+		.withRule(std::move(rule))
+		.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule float_value_works {
+	condition:
+		0.0
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule float_value_works
+{
+	condition:
+		0.0
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+IequalsWorks) {
+	auto cond = id("pe").access("pdb_path").iequals(stringVal("C:\\PATH")).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("iequals_builder")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withModule("pe")
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(import "pe"
+
+rule iequals_builder {
+	condition:
+		pe.pdb_path iequals "C:\\PATH"
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(import "pe"
+
+rule iequals_builder
+{
+	condition:
+		pe.pdb_path iequals "C:\\PATH"
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+IdWildcardWorks) {
+	auto cond = of(intVal(2), set({id("rule1"), id("ru*")})).get();
+
+	YaraRuleBuilder newRule;
+	auto refRule = newRule
+			.withName("rule1")
+			.withCondition(boolVal(true).get())
+			.get();
+
+	auto rule = newRule
+			.withName("of_rule_builder")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withRule(std::move(refRule))
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule rule1 {
+	condition:
+		true
+}
+
+rule of_rule_builder {
+	condition:
+		2 of (rule1, ru*)
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule rule1
+{
+	condition:
+		true
+}
+
+rule of_rule_builder
+{
+	condition:
+		2 of (rule1, ru*)
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+StringAsBoolInConditionWorks) {
+	auto cond = (stringVal("abc") && stringVal("def")).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("rule1")
+			.withCondition(std::move(cond))
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(rule rule1 {
+	condition:
+		"abc" and "def"
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(rule rule1
+{
+	condition:
+		"abc" and
+		"def"
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+IcontainsWorks) {
+	auto cond = id("pe").access("pdb_path").icontains(stringVal("C:\\PATH")).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("icontains_builder")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withModule("pe")
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(import "pe"
+
+rule icontains_builder {
+	condition:
+		pe.pdb_path icontains "C:\\PATH"
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(import "pe"
+
+rule icontains_builder
+{
+	condition:
+		pe.pdb_path icontains "C:\\PATH"
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+IendsWithWorks) {
+	auto cond = id("pe").access("pdb_path").iendswith(stringVal("C:\\PATH")).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("iendswith_builder")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withModule("pe")
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(import "pe"
+
+rule iendswith_builder {
+	condition:
+		pe.pdb_path iendswith "C:\\PATH"
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(import "pe"
+
+rule iendswith_builder
+{
+	condition:
+		pe.pdb_path iendswith "C:\\PATH"
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+IstartsWithWorks) {
+	auto cond = id("pe").access("pdb_path").istartswith(stringVal("C:\\PATH")).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("istartswith_builder")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withModule("pe")
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(import "pe"
+
+rule istartswith_builder {
+	condition:
+		pe.pdb_path istartswith "C:\\PATH"
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(import "pe"
+
+rule istartswith_builder
+{
+	condition:
+		pe.pdb_path istartswith "C:\\PATH"
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+StartsWithWorks) {
+	auto cond = id("pe").access("pdb_path").startswith(stringVal("C:\\PATH")).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("startswith_builder")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withModule("pe")
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(import "pe"
+
+rule startswith_builder {
+	condition:
+		pe.pdb_path startswith "C:\\PATH"
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(import "pe"
+
+rule startswith_builder
+{
+	condition:
+		pe.pdb_path startswith "C:\\PATH"
+}
+)", yaraFile->getTextFormatted());
+}
+
+TEST_F(BuilderTests,
+EndsWithWorks) {
+	auto cond = id("pe").access("pdb_path").endswith(stringVal("C:\\PATH")).get();
+
+	YaraRuleBuilder newRule;
+	auto rule = newRule
+			.withName("endswith_builder")
+			.withCondition(cond)
+			.get();
+
+	YaraFileBuilder newFile;
+	auto yaraFile = newFile
+			.withModule("pe")
+			.withRule(std::move(rule))
+			.get(true);
+
+	ASSERT_NE(nullptr, yaraFile);
+	EXPECT_EQ(R"(import "pe"
+
+rule endswith_builder {
+	condition:
+		pe.pdb_path endswith "C:\\PATH"
+})", yaraFile->getText());
+
+	EXPECT_EQ(R"(import "pe"
+
+rule endswith_builder
+{
+	condition:
+		pe.pdb_path endswith "C:\\PATH"
+}
+)", yaraFile->getTextFormatted());
+}
+
 }
 }
